const db = require('../db');
const authwrap = require('../authwrap');
const check_permission = require('../check_permission');

async function send_message(msg, respond) {
	if (!msg.thread) {
		return respond({
			success: false,
			message: 'thread ID required'
		});
	}
	if (!(await check_permission(msg.auth_user.id, msg.thread)).post) {
		return respond({
			success: false,
			message: "you can't post to this thread"
		});
	}
	// add message and send it to everyone
	const id = (await db.query(
		'insert into post (user, thread, content, encrypted) values (?, ?, ?, ?) returning id',
		[msg.auth_user.id, msg.thread, msg.message, msg.encrypted]
	)).rows[0].id;
	// get thread members
	const members = (
		await db.query('select user from member where member.thread = ?', msg.thread)
	).rows.map(i => i.user);
	// get perms
	const permissions = await db.query(
		`select * from permission
		where thread = ? and type = 'everyone' and value = 'true' and permission = 'view'`,
		msg.thread);
	for (let id in vybe.users) {
		if (permissions.rows.length > 0 || members.includes(id)) {
			for (let s of vybe.users[id].sockets) {
				s.emit('new_message', {
					id,
					userid: msg.auth_user.id,
					name: msg.auth_user.displayname,
					content: msg.message,
					thread: msg.thread
				});
			}
		}
	}
	return respond({
		success: true,
		id
	});
}

async function get_history(msg, respond) {
	if (msg.before && isNaN(Number(msg.before))) {
		return respond({
			success: false,
			message: 'before must be a number'
		});
	}
	if (!msg.thread) {
		return respond({
			success: false,
			message: 'thread ID required'
		});
	}
	if (!(await check_permission(msg.auth_user.id, msg.thread)).view) {
		return respond({
			success: false,
			message: "you can't view this thread"
		});
	}
	const messages = (await db.query(
		`select coalesce(displayname, name) as name, user as userid, post.id, content
		from post
		left join user on post.user = user.id
		where ${msg.before ? 'post.id < ? and' : ''}
		thread = ?
		order by post.created desc
		limit 101`,
		msg.before ? [msg.before, msg.thread] : [msg.thread]
	)).rows;
	return respond({
		success: true,
		messages: messages.slice(0, 100),
		more: messages.length > 100
	});
}

module.exports = {
	send_message: authwrap(send_message),
	get_history: authwrap(get_history)
};
